import { useReducer } from 'react';

type Key = string | number | symbol;

interface State<K extends Key, V> {
  map: Map<K, V>;
}
interface Action<K extends Key, V> {
  type: 'set' | 'delete' | 'clear';
  key?: K;
  value?: V;
}

function reducer<K extends Key, V>(state: State<K, V>, action: Action<K, V>) {
  switch (action.type) {
    case 'set': {
      state.map.set(action.key!, action.value!);
      break;
    }
    case 'delete': {
      state.map.delete(action.key!);
      break;
    }
    case 'clear': {
      state.map.clear();
      break;
    }
    default:
      break;
  }
  return {
    map: state.map,
  };
}

export function useMap<K extends Key, V extends number>(arr: [K, V][] = []) {
  const [state, dispatch] = useReducer(reducer, arr, (a) => ({
    map: new Map<K, V>(a),
  }));

  return {
    get: (key: K) => state.map.get(key),
    set: (key: K, value: V) => dispatch({ type: 'set', key, value }),
    delete: (key: K) => dispatch({ type: 'delete', key }),
    clear: () => dispatch({ type: 'clear' }),
    forEach: (callback: (value: V, key: K, set: Map<K, V>) => void) =>
      // @ts-expect-error ignore
      state.map.forEach(callback),
    has: (key: K) => state.map.has(key),
    entries: state.map.entries,
    values: state.map.values,
    keys: state.map.keys,
    size: state.map.size,
  };
}
